<?php

$params = require __DIR__ . '/params.php';
$db = require __DIR__ . '/db.php';
$routes = require __DIR__ . '/routes.php';
$config = [
    'id' => 'basic',
    'basePath' => dirname(__DIR__),
    'bootstrap' => ['log','rss','sitemap'],
    'language' => 'ru',
    'aliases' => [
        '@bower' 	=> '@vendor/bower-asset',
        '@npm'   	=> '@vendor/npm-asset',
        '@manager'	=> '@app/modules/manager',
        '@control'	=> '@app/modules/control',
        '@rss'		=> '@app/modules/rss',
        '@sitemap'	=> '@app/modules/sitemap',
    ],
	'timeZone' => 'Asia/Novosibirsk',
    'components' => [
        'request' => [
            // !!! insert a secret key in the following (if it is empty) - this is required by cookie validation
            'cookieValidationKey' => 'WyEyjj5Lk_8A5msMn1mn5-UQN_xDaNsc',
        ],
		'ya' => [
			'class' => 'app\models\Ya',
			'tokenAI' => 't1.9euelZqKz8-YxsyZyJWXi8eblM-Yle3rnpWal56NkcjMyMicipSeyp3LjInl9Pc_PBlO-e8Hd3bj3fT3f2oWTvnvB3d2483n9euelZqMlZfKioqbz87Kj4nOlonLmu_8xeuelZqMlZfKioqbz87Kj4nOlonLmg.vGi2BEtkZdD8WnuEzZJ8-uQwHECXS3ittjcv0ao_UUogKkhFw4cCiU_XVw8OjpLIh46xTGTzUQ9H5bCxM6tUCw',
			'tokenMetrica' => 'y0_AgAAAAABspRzAAKENgAAAADl049iHuuiZPWuSTOGV3NsbVUNsXCymlc',
			'idMetrica' => 34950850
		],
        'deviceDetect' => [
            'class' => 'alexandernst\devicedetect\DeviceDetect'
        ],
        'cache' => [
            'class' => 'yii\caching\FileCache',
        ],
        'memcache' => [
            'class' => 'yii\caching\MemCache',
			'useMemcached' => true,
            'servers' => [
                [
                    'host' => '127.0.0.1',
                    'port' => 11211,
                    'weight' => 100,
                ],
            ],
        ],
        'acache' => [
            'class' => 'yii\caching\ArrayCache',
        ],
        'user' => [
            'identityClass' => 'app\models\User',
            'enableAutoLogin' => true,
			'on afterLogin' => function($event) {
					$user = $event->identity;
					$user->last_login_at = time();
					$user->save();
			}
        ],
        'errorHandler' => [
            'errorAction' => 'site/error',
        ],
        'mailer' => [
            'class' => 'yii\swiftmailer\Mailer',
            // send all mails to a file by default. You have to set
            // 'useFileTransport' to false and configure transport
            // for the mailer to send real emails.
            'useFileTransport' => false,
        ],
        'log' => [
            'traceLevel' => YII_DEBUG ? 3 : 1,
            'targets' => [
                [
                    'class' => 'yii\log\FileTarget',
                    'levels' => ['error'],
                    'except' => ['yii\web\HttpException:404'],
                ],
            ],
        ],
        'db' => $db,
        'old_db' => require __DIR__ . '/db_old.php',
        'urlManager' => [
            'enablePrettyUrl' => true,
            'showScriptName' => false,
            'rules' =>$routes,
        ],
		'assetManager' => [
            'bundles' => [
                'yii\web\JqueryAsset' => [
                    'jsOptions' => ['position' => \yii\web\View::POS_HEAD],
					'js' => ['jquery.min.js']
                ],
                'yii\web\YiiAsset' => [
                    'jsOptions' => ['position' => \yii\web\View::POS_HEAD]
                ],
                'yii\bootstrap4\BootstrapAsset' => [
                   // 'cssOptions' => ["rel"=>"preload", "as"=>"style"],
					'css' => ['css/bootstrap.min.css']
                ],

            ],
        ],
		'sphinx' => [
            'class' => 'yii\sphinx\Connection',
            'dsn' => 'mysql:host=192.168.1.3;port=9306;',
            'username' => '',
            'password' => '',
        ],
        'og'=>[
            'class'=>\app\helpers\OG::class
        ],
		'view' => [
         'theme' => [
              'pathMap' => [
                  '@app/views' => [
                       '@dektrium/user/views'
                  ],
                  '@dektrium/user/views' => '@manager/views/user'
              ],
         ],
     ],
    ],
    "modules"=>[
		"manager"=>[
			"class"=>manager\Manager::class
		],
		"control"=>[
			"class"=>control\Control::class
		],
		"sitemap"=>[
			"class"=>sitemap\Sitemap::class,
      ],
      "rss" => [
        "class"=>rss\Rss::class,
        'supportModels'  => [ // list of supported models for displaying a Turbo-pages feed
            'News' => '\app\models\News',
        ],
        'cacheExpire' => 1, // cache lifetime, `0` - for not use cache
        'channelOptions' => ['title' => 'ИА Амител', 'link' => 'https://www.amic.ru/'], // default channel options
        'turboRoute' => '/rss', // default route to render Turbo-pages feed (use "/" - for root)
        'layout' => '@manager/views/layouts/main',
		'routePrefix' => ''
      ],
      'user' => [
            'class' => 'dektrium\user\Module',
            'admins' => ['admin'],
            'adminPermission' => 'admin',
			'mailer' => [ 'sender' => 'info@api.amic.ru', ],
            'controllerMap' => [
                'admin' => [
                    'class' => 'dektrium\user\controllers\AdminController',
                    'layout' => '@manager/views/layouts/main'
                ],
                'profile' => [
                    'class' => 'dektrium\user\controllers\ProfileController',
                    'layout' => '@manager/views/layouts/main'
                ],
            ]
      ],
      'rbac' => [
            'class' => 'dektrium\rbac\RbacWebModule',
            'controllerMap' => [
                'role' => [
                    'class' => 'dektrium\rbac\controllers\RoleController',
                    'layout' => '@manager/views/layouts/main'
                ],
                'permission' => [
                    'class' => 'dektrium\rbac\controllers\PermissionController',
                    'layout' => '@manager/views/layouts/main'
                ],
                'rule' => [
                    'class' => 'dektrium\rbac\controllers\RuleController',
                    'layout' => '@manager/views/layouts/main'
                ]
            ]
      ]
    ],
    'params' => $params,
];

if (YII_ENV_DEV) {
    // configuration adjustments for 'dev' environment
    $config['bootstrap'][] = 'debug';
    $config['modules']['debug'] = [
        'class' => 'yii\debug\Module',
        // uncomment the following to add your IP if you are not connecting from localhost.
        'allowedIPs' => ['*'],
    ];

    $config['bootstrap'][] = 'gii';
    $config['modules']['gii'] = [
        'class' => 'yii\gii\Module',
        // uncomment the following to add your IP if you are not connecting from localhost.
        'allowedIPs' => ['*'],
    ];
}
Yii::setAlias('@manager',  '../modules/manager');
return $config;
